#coding:utf-8
'''
Created on 25.05.2011

@author: prefer
'''

import threading
import inspect

from django.conf import settings
from django.utils.importlib import import_module

try:
    from settings import EXCLUDE_APPS
except ImportError:
    EXCLUDE_APPS = []

class ReportGeneratorClientError(Exception):
    """
    Некая ошибка на клиенте генератора отчетов.
    """


class EntityCache(object):
    '''
    Загружает и хранит список определенных сущностей в проекте
    '''
    
    _loaded = False
    _write_lock = threading.RLock()
    
    # Список сущностей
    _entities = []
    
    # Список классов для произвольных данных 
    _data_classes = []
    
    @classmethod
    def populate(cls):
        '''
        Собирает в отдельном потоке все сущности и наборы данных из установленных приложений с 
        названием schema
        '''
        if not cls._loaded:
            with cls._write_lock:
                for app_name in settings.INSTALLED_APPS:
                    if app_name not in EXCLUDE_APPS:
                        try:
                            module = import_module('.schema', app_name)
                        except ImportError, err:
                            if 'No module named' not in err.args[0]:
                                raise
                            continue
                                        
                        entities_module = []
                        data_class = []

                        queue = []
                        for _, obj in inspect.getmembers(module):
                            # Флаг. = True, если сущность основана только на Django моделях
                            #       = False, если сущность основана хотя бы на одной другой сущности
                            entity_based_on_django_model = True
                            if inspect.isclass(obj) and hasattr(obj, '__module__') and obj.__module__ == module.__name__:
                                # Чтобы не было кроссимпорта!
                                if 'BaseEntity' in [x.__name__ for x in obj.__bases__]:
                                    for entity in obj().entities:
                                        if type(entity).__name__ == 'Entity':
                                            entity_based_on_django_model = False

                                    if entity_based_on_django_model:
                                        entities_module.append(obj)
                                    else:
                                        queue.append(obj)
                                
                                # Чтобы не было кроссимпорта!
                                if 'BaseData' in [x.__name__ for x in obj.__bases__]:
                                    data_class.append(obj)

                        while queue:
                            entity = queue.pop(0)
                            entities_module.append(entity)
                    
                        cls._entities.extend(entities_module)
                        cls._data_classes.extend(data_class)

                    cls._loaded = True

    @classmethod        
    def get_entities(cls):
        '''
        Возвращает список сущностей
        '''
        cls.populate()        
        return cls._entities
    
    @classmethod
    def get_entity(cls, name):
        '''
        Возвращает сущность по имени класса
        '''
        cls.populate()
        for entity in cls._entities:
            if entity.__name__ == name:
                return entity
        
    @classmethod        
    def get_data_classes(cls):
        '''
        Возвращает список классов для произвольных данных
        '''
        cls.populate()        
        return cls._data_classes
    
    @classmethod
    def get_data_class(cls, name):
        '''
        Возвращает класс произвольных данных
        '''
        cls.populate()
        for cl in cls._data_classes:
            if cl.__name__ == name:
                return cl