#coding:utf-8
import socket

import urllib
import urllib2
import urlparse
import json
from django.core.exceptions import ObjectDoesNotExist
from django.http import HttpResponse
from django.conf import settings

from m3.helpers.icons import Icons

from m3.ui.ext.containers import containers
from m3.ui.ext.controls import buttons
from m3.ui.ext.fields import simple
from m3.ui.ext.fields.complex import ExtDictSelectField
from m3.ui.ext.misc import store

from m3.ui import actions
from m3.ui.actions import results
from m3_query_builder.api import MetadataChecker, synchronize_parameters

from m3_query_builder.entity import Param
from m3_query_builder.enums import ReportType
from m3_query_builder.models import ReportReference, FormField
from m3_query_builder.ui import ReportGeneratorWindow
from m3_query_builder import (metadata, ReportGeneratorClientError, api, ui,
    models)

from m3.helpers import logger
from m3.ui.actions.results import OperationResult


class ReportGeneratorPack(actions.ActionPack):
    """
    Пак для GetReportGeneratorAction, без пака никак :(
    """
    url = '/m3-report-generator'

    def __init__(self):
        super(ReportGeneratorPack, self).__init__()        
        self.actions.extend([
            GetReportGeneratorAction(),
            SynchronizeAction(),
        ])
        

class GetReportGeneratorAction(actions.Action):
    """
    Загружает в iframe генератор отчетов
    """
    url = '/get'
    
    def run(self, request, context):
        # Отправление запроса на метаданные, сначала с кэшем, если кэш устарел,
        # то отправление метаданных - получение положительного ответа
        # после сохранения. После этого загрузка в iframe
        # Необходимо в settings.py прописать строку подключения где запущен
        # генератор отчетов, вида:
        # REPORT_GENERATOR_URL = http://localhost:8000/
        
        def create_window():
            window = ReportGeneratorWindow(
                src=urlparse.urljoin(
                    metadata.ReportGeneratorWorker.get_report_generator_url(),
                    '/?hash=%s' % metadata.ReportGeneratorWorker.get_metadata_hash()))

            window.synchronize_url = SynchronizeAction.absolute_url()

            return results.ExtUIScriptResult(data=window)

        try:        
            if MetadataChecker.synchronize():                
                return create_window()
            else:
                return results.OperationResult(success=False)
        except ReportGeneratorClientError, e:
            return results.OperationResult(success=False, message=e.message)
        except urllib2.URLError:            
            logger.exception()
            return results.OperationResult(success=False,
                message=u'Сервер генератора отчетов не отвечает')


class SynchronizeAction(actions.Action):
    """
    Синхронизация с генератором отчетов и обновление параметров
    """
    url = '/synchronize'
    shortname = 'report-params-synchronize'
    
    report_synchr_url = 'reports/get-reports-params'
    
    def run(self, request, context):

        system_hash = metadata.ReportGeneratorWorker.get_metadata_hash()
        
        url = urlparse.urljoin(
            metadata.ReportGeneratorWorker.get_report_generator_url(), 
            self.report_synchr_url)
                
        report_form_request = urllib2.urlopen(url,
            data=urllib.urlencode({'system_hash': system_hash,}))

        result = json.loads(report_form_request.read())
        if not result['success']:
            return OperationResult(success=False, message=result.get('message'))
                
        synchronize_parameters(result['reports'])
        
        return OperationResult()
        
class ReportFormsActionsPack(actions.ActionPack):
    '''
    Экшенпак работы с формами отчетов
    '''
    url = '/report-forms'
    shortname = 'report-forms-builder'

    def __init__(self):
        super(ReportFormsActionsPack, self).__init__()

        self.list_window_action = GetReportFormsWindow()

        self.actions.extend([
            self.list_window_action,
            GetReportForm(),
            ListReportForms(),
            PassReportFormToReportGenerator(),
        ])

    def get_list_url(self):
        '''
        Возвращает адрес формы списка элементов справочника.
        Используется для присвоения адресов в прикладном приложении.
        '''
        return self.list_window_action.get_absolute_url()


class GetReportFormsWindow(actions.Action):
    """
    Возвращает окно со списком отчетов
    """
    url = '/window'
    shortname = 'get_report_forms_window_action'

    def run(self, request, context):
        window = ui.ReportFormListWindow()
        return results.ExtUIScriptResult(window)
    

class ListReportForms(actions.Action):
    """
    Список форм отчетов
    """
    url = '/list'
    shortname = 'list_report_forms_action'

    def run(self, request, context):
        form_fields = list(models.ReportReference.objects.all())
        return results.PreJsonResult({'rows':form_fields,
            'total':len(form_fields)})


class GetReportForm(actions.Action):
    '''
    Возвращает форму отчета
    '''
    url = '/report-form'
    shortname = 'report_form_action'

    def context_declaration(self):
        return [actions.ACD(name='id', type=str, required=True,
                    verbose_name=u'Идентификатор отчета'),]

    def _get_field(self, param):
        if param['type'] == Param.STRING:
            field = simple.ExtStringField()
        elif param['type'] == Param.NUMBER:
            field = simple.ExtNumberField()
        elif param['type'] == Param.DATE:
            field = simple.ExtDateField(hide_today_btn=True)
        elif param['type'] == Param.BOOLEAN:
            field = simple.ExtCheckBox()
        elif param['type'] == Param.DICTIONARY:

            # Здесь нужно проверять pack на возможность множественного
            # выбора и если такой возможен, делать множественный выбор

            field = ExtDictSelectField(hide_edit_trigger=True)

            if not param['type_value']:
                return

            field.pack = param['type_value']

        elif param['type'] == Param.NUMBER:
            field = simple.ExtNumberField()
        elif param['type'] == Param.COMBO:
            field = simple.ExtComboBox(display_field='text',
                value_field='id', trigger_action=simple.ExtComboBox.ALL)
            data = api.get_data_class(param['type_value']).get_data()
            field.store = store.ExtDataStore(data)
        else:
            raise Exception('type "%s" is not define in class TypeField'
                % param['type'])

        return field

    def _get_report_params(self, report_id):
        """
        Возвращает параметры отчета.

        @raise ReportGeneratorClientError
        """
        try:
            reference = ReportReference.objects.get(id=report_id)
        except ObjectDoesNotExist:
            raise ReportGeneratorClientError(u"Отчет не найден. "
                u"Скорее всего он был удален при синхронизации. Попробуйте "
                u"обновить список отчетов.")

        name = reference.name
        document_type = reference.document_type

        generator_report_id = reference.report_id

        params = FormField.objects.filter(report__id=report_id).values(
            'id', 'verbose_name', 'type', 'type_value', 'required',
            'multi_choice', 'default_value')

        return dict(name=name, params=params,
            generator_report_id=generator_report_id, document_type=document_type)

    def run(self, request, context):
        win = ui.ReportForm()
        win.submit_data_url = PassReportFormToReportGenerator.absolute_url()

        try:
            report_params = self._get_report_params(context.id)
        except ReportGeneratorClientError, e:
            return results.OperationResult(success=False, message=unicode(e))

        name = report_params['name']
        params = report_params['params']
        generator_report_id = report_params['generator_report_id']
        document_type = report_params['document_type']

        win.title = name
        win.hdn_report_id.value = generator_report_id
        win.frm_params.layout_config={'align':'stretch'}

        win.set_format_combobox_store(ReportType.get_type_enum(document_type))

        multiple_choice = {}

        for param in params:
            field = self._get_field(param)
            if not field:
                continue

            field.anchor = '100%'
            field.label = param['verbose_name']
            field.name = param['id']
            field.enable_key_events = True

            cont_outer = containers.ExtContainer(layout='hbox')
            cont_inner = containers.ExtContainer(layout='form', flex=1)

            cont_outer.items.append(cont_inner)
            cont_inner.items.append(field)

            if param['type'] in (Param.STRING, Param.NUMBER, Param.DATE,
                Param.COMBO):
                cont_outer.items.append(buttons.ExtButton(
                    handler='function(){ addValue("%s");}' % field.client_id,
                    icon_cls=Icons.ADD, client_id='btn-'+field.client_id,))

                if param['type'] in (Param.STRING, Param.NUMBER):
                    field.value = param['default_value']

            win.frm_params.items.append(cont_outer)

        win.multiple_choice = json.dumps(multiple_choice)

        return results.ExtUIScriptResult(win)


class PassReportFormToReportGenerator(actions.Action):
    """
    Получает параметры из формы отчета, и пробрасывает их дальше на сервер
    генератора отчетов

    Attributes:
        id: Ключ отчета
        params: Параметры из формы отчета.
    """
    url = '/to_report_generator'
    shortname = 'pass_report_form_to_report_generator_action'

    def context_declaration(self):
        return [
            actions.ACD(name='id', required=True, type=int),
            actions.ACD(name='params', required=True, type=object)]

    def run(self, request, context):
        try:
            if not MetadataChecker.check_hash():
                return results.OperationResult(success=False,
                    message=u'Метаданные системы изменились!')
        except ReportGeneratorClientError:
            return results.OperationResult(success=False,
                message=u'Не удалось соединиться с сервером')


        url = urlparse.urljoin(
            metadata.ReportGeneratorWorker.get_report_generator_url(),
            'reports/receive_form')

        params = {}

        form_fields = models.FormField.objects.select_related().filter(
            id__in=[key for key in context.params.keys()
                # TODO(Excinsky): Если параметры формы будут добавляться,
                # нужно будет вынести их в перечисление, чтобы постоянно
                # не добавлять сюда
                if key not in ['report_id', 'result_format']])
        
        for field in form_fields:
            for param in field.parameter_set.all():
                if field.required and not context.params[str(field.id)]:
                    return OperationResult(success=False, message=u'Не задан обязательный параметр %s' % param.name)
                params[param.name] = context.params[str(field.id)]

        try:
            report_form_request = urllib2.urlopen(url,
                data=urllib.urlencode({
                    'hash': metadata.ReportGeneratorWorker.get_metadata_hash(),
                    'report_id': context.params['report_id'],
                    'result_format': context.params['result_format'],
                    'params': json.dumps(params),
                    'system_id': settings.REPORT_GENERATOR_SYSTEM_ID,
                    'for_data': False
                }), timeout=60)
        except urllib2.URLError, e:
            message=u'На сервере произошла непонятная ошибка. '
            if hasattr(e, 'reason') and e.reason.message == 'timed out':
                message = u'Недостаточно времени для формирования отчёта. Увеличьте время отклика. '
            return results.OperationResult(success=False,
                message=message)

        json_result = report_form_request.read()

        report_form_request.close()

        result = json.loads(json_result)

        message = result.get('message')
        if message:
            return results.OperationResult(success=False,
                message=message)

        success = result.get('success')
        if success:
            static_file_path = result.get('static_file_path')
            if not static_file_path:
                return results.OperationResult(success=False,
                    message=u'Сервер сообщил об успешной генерации отчета,'
                        u'но не прислал ссылку на него.')
            else:
                result['static_file_path'] = urlparse.urljoin(
                    metadata.ReportGeneratorWorker.get_report_generator_url(),
                    result['static_file_path'])
                return HttpResponse(content=json.dumps(result),
                    mimetype='application/json')

        return results.OperationResult(success=False,
                message=u'Проблемы на сервере генератора отчетов')

            