
#coding: utf-8

'''
Created on 06.06.2011

@author: prefer
'''
import urllib
import urllib2
import urlparse
import json

from django.http import HttpResponse
from django.conf import settings

from m3.ui.actions import results

from m3_query_builder import (metadata, ReportGeneratorClientError,
    EntityCache)
from m3_query_builder.models import ReportReference, FormField, Parameter
from m3_query_builder.metadata import ReportGeneratorWorker
from m3_query_builder.enums import TableDocumentResultFileTypes

##==============================================================================
## Вспомогательные функции для получения различной информации о запросе
def get_group_fields(objs, separator='-'):
    '''
    Возвращает поля группировок
    '''
    if group_fields_exist(objs):
        for group_field in objs['group']['group_fields']:
            entity_name, field_name = group_field['id'].split(separator)
            yield entity_name, field_name
    else:
        yield '', ''

def group_fields_exist(objs):
    """
    Существуют ли поля группировки?

    @param objs: json-представление запроса
    @return: bool
    """
    return bool(objs.get('group'))

def get_limit(objs):
    '''
    Возвращает лимит для запроса
    '''
    return objs['limit']

def get_sorted_fields(objs, separator='-'):
    '''
    Возвращает поля, по которым требуется сортировка
    '''
    if select_fields_exist(objs):
        return [field['fieldName'] for field in objs['selected_fields'] if field.get('sorting')]
    else:
        return []

def select_fields_exist(objs):
    """
    Проверяет существование полей, которые должны оказаться в SELECT'е

    @param objs: json-представление запроса
    @return: bool
    """
    return bool(objs.get('selected_fields'))


def get_report(id):
    '''
    Возвращает прикрепленный отчет по id
    '''
    return ReportReference.objects.get(id=id)

def get_data_class(class_name):
    '''
    Возвращает класс для данных по имени класса
    '''
    cl = EntityCache.get_data_class(class_name)
    assert cl, 'cl is not defined in EntityCache'
    return cl

class MetadataChecker(object):
    """
    Проверяет состояние синхронизации метаданных
    """
    # 20 секунд для того, чтобы понять, что сервер отчетов не отвечает
    DEFAULT_TIMEOUT = 20
    
    @classmethod
    def _send_metadata_hash(cls):
        """
        Отправляет хэш метаданных на сервер, чтобы убедиться, что они не успели
        измениться со времени предыдущего общения клиента и сервера
        """
        try:
            url = urlparse.urljoin(
                metadata.ReportGeneratorWorker.get_report_generator_metadata_url(),
                    'open')
            metadata_request = urllib2.urlopen(url, data=urllib.urlencode({
                    'system_id': metadata.ReportGeneratorWorker.get_system_id(),
                    'metadata_hash':
                        metadata.ReportGeneratorWorker.get_metadata_hash()}),

                timeout=cls.DEFAULT_TIMEOUT
            )
        except urllib2.URLError:
            raise ReportGeneratorClientError(u"Не удалось соединиться с сервером")

        json_result = metadata_request.read()
        metadata_request.close()
        result = json.loads(json_result)

        message = result.get('message')
        if message:
            raise ReportGeneratorClientError(result.get('message'))

        return result.get('success')

    @classmethod
    def _send_metadata(cls):
        u"""
        Отправляет метаданные с хешем для сохранения на сервер

        @raise ReportGeneratorClientError
        """
        metadata_request = urllib2.urlopen(urlparse.urljoin(
            metadata.ReportGeneratorWorker.get_report_generator_metadata_url(),
                'metadata'),
            data=urllib.urlencode({
                'system_id': metadata.ReportGeneratorWorker.get_system_id(),
                'metadata_json':
                    metadata.ReportGeneratorWorker.get_json_metadata()}),
            timeout=cls.DEFAULT_TIMEOUT
        )

        json_result = metadata_request.read()
        metadata_request.close()
        result = json.loads(json_result)

        success = result.get('success')
        if success:
            return True
        else:
            raise ReportGeneratorClientError(result.get('message'))

    @classmethod
    def check_hash(cls):
        u"""
        Проверяет состояние хеша метаданных системы

        @return: bool
        """
        return bool(cls._send_metadata_hash())

    @classmethod
    def synchronize(cls):
        u"""
        Синхронизирует метаданные

        @raise: urllib2.URLError, ReportGeneratorClientError
        @return: bool
        """
        try:            
            if cls.check_hash():
                return True
            else:                
                send_result = cls._send_metadata()                
                if send_result:
                    return True
                else:
                    return False
        except urllib2.URLError:
            raise
        except ReportGeneratorClientError:
            raise


def synchronize_parameters(reports):
    """
    Синхронизирует параметры
    """
    # Удаляем все что есть, каскадно
    ReportReference.objects.all().delete()
    for report in reports:        
        r = ReportReference(report_id=report['id'], name=report['name'],
            document_type=int(report['document_type']))
        r.save()
        
        for form_field in report['form_fields']:            
            field = FormField(report=r)
            field.verbose_name = form_field['verbose_name']
            field.required = form_field['required']
            field.multi_choice = form_field['multi_choice']
            field.type = form_field['type']
            field.type_value = form_field['type_value']
            field.default_value = form_field['default_type']
            field.save()
            
            for param in form_field['parameters']:
                Parameter(field=field, name=param).save()

def build_report(report_key, params):
    """
    Метод формирует отчет, построенный в КО.
    Отчет задается по system_id
    @params - словарь параметров отчета
    """

    url = urlparse.urljoin(ReportGeneratorWorker.get_report_generator_url(), 'reports/receive_form')

    try:
        report_form_request = urllib2.urlopen(url,
            data=urllib.urlencode({
                'report_key': report_key,
                # Здесь жестко проставляем тип результата, т.к. пока доступно только XLSX.
                'result_format': TableDocumentResultFileTypes.XLSX,
                'params': params,
                'system_id': settings.REPORT_GENERATOR_SYSTEM_ID,
                'for_data': False
            }), timeout=60)
    except urllib2.URLError, e:
        message=u'На сервере произошла непонятная ошибка. '
        if getattr(e, 'reason', None) and e.reason.message == 'timed_out':
            message = u'Недостаточно времени для формирования отчёта. Увеличьте время отклика. '
        return results.OperationResult(success=False,
            message=message)

    json_result = report_form_request.read()
    report_form_request.close()
    result = json.loads(json_result)

    message = result.get('message')
    if message:
        return results.OperationResult(success=False,
            message=message)

    success = result.get('success')
    if success:
        static_file_path = result.get('static_file_path')
        if not static_file_path:
            return results.OperationResult(success=False,
                message=u'Сервер сообщил об успешной генерации отчета,'
                        u'но не прислал ссылку на него.')
        else:
            result['static_file_path'] = urlparse.urljoin(
                metadata.ReportGeneratorWorker.get_report_generator_url(),
                result['static_file_path'])
            return HttpResponse(content=json.dumps(result),
                mimetype='application/json')

    return results.OperationResult(success=False,
        message=u'Проблемы на сервере генератора отчетов')


def get_report_data(report_key, params):
    """
    Функция возвращает данные для отчета, соответствующего ключу report_key
    """

    url = urlparse.urljoin(ReportGeneratorWorker.get_report_generator_url(), 'reports/receive_form')

    try:
        report_form_request = urllib2.urlopen(url,
            data=urllib.urlencode({
                'report_key': report_key,
                # Здесь жестко проставляем тип результата, т.к. пока доступно только XLSX.
                'result_format': TableDocumentResultFileTypes.XLSX,
                'params': params,
                'system_id': settings.REPORT_GENERATOR_SYSTEM_ID,
                'for_data': True
            }), timeout=60)
    except urllib2.URLError, e:
        message=u'На сервере произошла непонятная ошибка. '
        if getattr(e, 'reason', None) and e.reason.message == 'timed_out':
            message = u'Недостаточно времени для формирования отчёта. Увеличьте время отклика. '
        return results.OperationResult(success=False,
            message=message)

    json_result = report_form_request.read()
    report_form_request.close()
    result = json.loads(json_result)

    return result
