#coding:utf-8
#@author: Excinsky
import hashlib
import urlparse
import json

from django.conf import settings

from objectpack import ObjectPack

from m3_query_builder import ReportGeneratorClientError, EntityCache
from m3_query_builder import entity as entity_mdl
from m3_query_builder.entity import Where, Param, Field

from m3.ui.actions import ControllerCache

from m3.ui.actions.dicts.simple import BaseDictionaryActions as newBDA
from m3.ui.actions.dicts.tree import BaseTreeDictionaryActions as newBTDA
from m3.ui.actions.packs import BaseDictionaryActions as oldBDA
from m3.ui.actions.tree_packs import BaseTreeDictionaryActions as oldBTDA
from m3.ui.actions.packs import BaseDictionaryModelActions as BDMA

import entity as entity_module


# NOTE(Excinsky): Невозможно оформить как декоратор, так как
# классметоды плохо их переваривают.
def check_metadata_settings():
    """
    Проверяет, существуют ли необходимые настройки

    @raise ReportGeneratorClientError
    """
    try:        
        settings.METADATA_URLS
        settings.METADATA_CONNECTION
        settings.REPORT_GENERATOR_URL
        settings.REPORT_GENERATOR_SYSTEM_ID
    except AttributeError:
        raise ReportGeneratorClientError(u'Неверные настройки клиента генератора отчетов. Пожалуйста, поправьте файл settings.py.')


class ReportGeneratorWorker(object):
    """
    Вспомогательный объект для работы с настройками генератора отчетов.
    """
    @classmethod
    def get_metadata_hash(cls):
        u"""
        Генерирует хэш на основе текущих настроек метаданных проекта
        """
        check_metadata_settings()
        hashh = hashlib.md5()        
        hashh.update(str(settings.METADATA_URLS))
        hashh.update(str(settings.METADATA_CONNECTION))
        hashh.update( json.dumps( cls.get_items_types() ) )
        # Entities представляют из себя классы, структуру которых нужно
        # учитывать. 
        for entity in EntityCache.get_entities():
            for elem in dir(entity):
                hashh.update(elem)

        hashh.update(ReportGeneratorWorker.get_system_id())

        return hashh.hexdigest()

    @classmethod
    def get_json_metadata(cls):
        u"""
        Генерирует JSON из текущих метаданных
        """
        #TODO: добавить собственно метаданные сюда
        check_metadata_settings()
        metadata = dict(            
            urls=settings.METADATA_URLS,
            connection=settings.METADATA_CONNECTION,
            item_types=cls.get_items_types(),
            hash=cls.get_metadata_hash()
        )

        metadata['metadata'] = ReportGeneratorWorker.__get_entities()

        return json.dumps(metadata)

    @classmethod
    def get_items_types(cls):
        """
        Возвращает типы полей, которые могут быть при генерации отчета
        """

        # При старте нового потока формируется коллекция паков.
        # Однако порядок элементов в коллекции, может быть вообще говоря произвольным.
        # Для того, чтобы хэш был устойчив от такого вида случайностей, необходимо отсортировать паки
        # по абсолятному урлу. Данная функция является критерием сортировки коллекции паков.
        def sort_pack_by_absolute_url(pack):
            return pack.get('absolute_url')

        res = []
        for key, value  in Param.get_type_choices():
            d = dict(type=key, verbose_name=value, type_values=[])
            if key == Param.DICTIONARY:
                # Собираем все паки
                # Обязательно сортируем, иначе проблемы с многопоточностью
                d['type_values'] = sorted(cls.get_packs(), key=sort_pack_by_absolute_url)
            res.append(d)                    
        return res

    @classmethod
    def get_packs(cls):
        """
        Возвращает все паки выбора из справочника в проекте
        """
        if not ControllerCache.get_controllers():
            ControllerCache.populate()

        for cont in ControllerCache.get_controllers():
            for pack in cont.get_packs():                
                if (isinstance(pack, newBDA) or isinstance(pack, newBTDA) or isinstance(pack, ObjectPack)
                    or isinstance(pack, oldBTDA) or isinstance(pack, oldBDA) or isinstance(pack, BDMA)):
                    yield {'name': unicode(pack.__class__.__name__),
                           'verbose_name': unicode(pack.verbose_name) or unicode(pack.title) or unicode(pack.__class__.__name__),
                           # Передаем абсолютный урл для сортировки паков.
                           'absolute_url': unicode(pack.absolute_url())}

    @classmethod
    def __validate_url(cls, url):
        pass

    @classmethod
    def get_report_generator_url(cls):
        """
        Возвращает URL сервера с генератором отчетов.
        """
        check_metadata_settings()
        return settings.REPORT_GENERATOR_URL

    @classmethod
    def get_report_generator_metadata_url(cls):
        """
        Возвращает URL до экшнов обработки метаданных.
        """
        check_metadata_settings()
        return urlparse.urljoin(settings.REPORT_GENERATOR_URL, 'metadata/')

    @classmethod
    def get_system_id(cls):
        """
        Возвращает ключ системы на сервере генератора отчетов.
        """
        check_metadata_settings()
        return settings.REPORT_GENERATOR_SYSTEM_ID

    @classmethod
    def generate_field(cls, field, entities, generate_for_entity = False):
        """
        Возвращает соответствущee полe или список всех полей, если требуется
        выбрать все поля (*)

        @param field: Field
        @return: dict or list
        """
        if field.get_field_name() == Field.ALL_FIELDS:
            fields = field.get_fields(entities)

            new_fields = fields if generate_for_entity else []
            if not generate_for_entity:
                new_fields = []
                for entity_field in fields:
                    new_fields.append(dict(table=entity_field.get_table_name(),
                        name=entity_field.get_table_field_name(),
                        verbose_name=entity_field.get_verbose_name()))
            return new_fields
        else:
            if generate_for_entity:
                for e in entities:
                    # Ищем entity, на которой основана текущая entity
                    if e['id'] == field.entity.name:
                        for select_field in e['select']:
                            # Ищем select-поле с таким же названием
                            if select_field['name'] == field.field_name:
                                return dict(table=select_field['table'],
                                    name=field.get_table_field_name() if not field.alias else field.alias,
                                    verbose_name=field.get_verbose_name())
            else:
                return dict(table=field.get_table_name(),
                    name=field.get_table_field_name(),
                    verbose_name=field.get_verbose_name())

    @classmethod
    def __get_entities(cls):

        def form_where_field(head, entities):
            dic = dict()
            if isinstance(head, entity_mdl.Field):
                dic['type'] = 'field'
                dic['field'] = cls.generate_field(head, entities)

            elif isinstance(head, entity_mdl.Param):
                dic['type'] = 'param'
                dic['param'] = dict(name=head.get_name(), type=head.get_type(),
                    verbose_name=head.get_verbose_name())

            elif isinstance(head, entity_mdl.FixedValue):
                dic['type'] = 'fixed_value'
                dic['fixed_value'] = dict(name=head.get_name(), type=head.get_type(),
                    verbose_name=head.get_verbose_name(), value=head.get_value())

            return dic

        def prepare_where_clause(where, entities):
            # Пустые условия пропускаем
            new_dict = dict()

            if not where:
                return {}

            # Если условие составное, то обрабатываем его рекурсивно
            left, right = where.left, where.right
            # Это XOR, для тех кто забыл дискретку
            if (isinstance(where.left, entity_mdl.Where) ==
                isinstance(where.right, entity_mdl.Where)):

                new_dict['type'] = 'where'
                new_dict['operator'] = where.operator
                if isinstance(where.left, entity_mdl.Where):
                    new_dict['left'] = prepare_where_clause(left, entities)
                    new_dict['right'] = prepare_where_clause(right, entities)
                else:
                    new_dict['left'] = form_where_field(left, entities)
                    new_dict['right'] = form_where_field(right, entities)

                return new_dict
            else:
                # Здесь останутся запросы, в которых одна часть это объект
                # Where, а другая это Field или Param. Такой запрос считается
                # неверно сформированным, и остается пустым.
                if where.operator != Where.NOT:
                    return {}
                #Однако, если это оператор not, то все норм
                new_dict['type'] = 'where'
                new_dict['operator'] = where.operator
                if isinstance(where.left, entity_mdl.Where):
                    new_dict['left'] = prepare_where_clause(left, entities)
                else:
                    new_dict['right'] = prepare_where_clause(right, entities)

            # Если отсутствует одна из частей условия, то возвращаем
            # существующую
            if right is not None:
                new_dict['left'] = form_where_field(right, entities)
            if left is not None:
                new_dict['right'] = form_where_field(left, entities)

            return new_dict

        entities = []
        for entity_cls in EntityCache.get_entities():
            entity = entity_cls()

            new_entity = dict(name=entity.name, id=entity_cls.__name__)
            new_entity['tables'] = []
            
            for table_entity in entity.get_entities():
                #assert isinstance(table_entity, entity_module.Model), u'Сущности не должны присутствовать в схемах'
                # Это Django моделька поэтому берём название таблицы у нее.
                if isinstance(table_entity, entity_module.Model):
                    new_entity['tables'].append(table_entity.get_table_name())
                # Это сущность. Вытаскиваем название из entities. Так как эта сущность уже была пройдена.
                else:
                    for e in entities:
                        if e['id'] == table_entity.name:
                            new_entity['tables'].extend(e['tables'])

            new_entity['relations'] = []

            for relation in entity.get_relations():
                new_relation = {
                    'first_field': cls.generate_field(relation.get_first_field(), entities),
                    'outer_first': relation.outer_first,
                    'second_field': cls.generate_field(relation.get_second_field(), entities),
                    'outer_second': relation.outer_second
                }
                new_entity['relations'].append(new_relation)

            new_entity['grouping'] = dict(group_fields=[], aggregation_fields=[])

            if isinstance(entity.get_group_by(), entity_mdl.Grouping):
                for group_field in entity.get_group_by().get_group_fields():
                    new_entity['grouping']['group_fields'].append(
                        cls.generate_field(group_field, entities))

                for aggregation_field in entity.get_group_by().get_aggregation_fields():
                    new_aggregation_field = {
                        'field':cls.generate_field(aggregation_field.get_field(), entities),
                        'mode': aggregation_field.get_aggregate_mode()
                    }
                    new_entity['grouping']['aggregation_fields'].append(
                        new_aggregation_field)

            new_entity['order_by'] = []
            for ordered_field in entity.get_order_by():
                new_ordered_field = {
                    'field':cls.generate_field(ordered_field.get_field(), entities),
                    'sort_order': ordered_field.get_sort_order()
                }
                new_entity['order_by'].append(new_ordered_field)

            new_entity['select'] = []
            for select_field in entity.get_select():
                generate_for_entity = True if isinstance(select_field.entity, entity_module.Entity) else False
                field_or_fields = cls.generate_field(select_field, entities, generate_for_entity)
                if isinstance(field_or_fields, list):
                    new_entity['select'].extend(field_or_fields)
                else:
                    new_entity['select'].append(field_or_fields)

            new_entity['distinct'] = entity.distinct
            new_entity['limit'] = entity.limit
            new_entity['offset'] = entity.offset

            new_entity['where'] = prepare_where_clause(entity.where, entities)

            entities.append(new_entity)

        return entities