#coding:utf-8 
'''
Date&Time: 01.06.11 10:28
@author: prefer
'''

from m3.helpers import urls
from m3.ui.ext import  render_component
from m3.ui.ext.all_components import *
from m3.ui.ext.panels.grids import ExtMultiGroupinGrid
from m3_query_builder.enums import DocumentResultFileTypes, TableDocumentResultFileTypes, ReportType



class ReportGeneratorWindow(ExtWindow):
    """
    """
    def __init__(self, src, *args, **kwargs):
        super(ReportGeneratorWindow, self).__init__(*args, **kwargs)
        self.template = 'report-generator.js' 
        self.src = src       
        self.synchronize_url = None
        
    def render(self):
        assert self.synchronize_url, 'synchronize_url is not defined'
        return render_component(self)


class ReportFormListWindow(ExtWindow):
    """
    """
    def __init__(self, *args, **kwargs):
        super(ReportFormListWindow, self).__init__(*args, **kwargs)
        self.layout = 'fit'
        self.title = u'Список доступных отчетов'
        self.width = 700
        self.height = 500
        self.template_globals = 'rb-report-list.js'

        #cross imports
        from m3_query_builder import actions

        self.report_form_url = actions.GetReportForm.absolute_url()
        self.buttons.insert(0, ExtButton(text=u'Показать форму отчета',
            handler='openReportForm'))

        cancel_btn = ExtButton(text=u'Закрыть')
        cancel_btn.handler = 'function(){ win.close ();}'

        self.buttons.extend((cancel_btn,))

        self.__init_grid()

    def __init_grid(self):
        grid = ExtObjectGrid()
        grid.add_column(header = u'Название', data_index = 'name')
        grid.row_id_name = 'report_id'
        grid.sm = ExtGridRowSelModel()
        grid.sm.single_select = True
        grid.action_data = urls.get_action('list_report_forms_action')

        self.reports_grid = grid

        self.items.append(self.reports_grid)

class ReportForm(ExtWindow):
    def __init__(self, *args, **kwargs):
        super(ReportForm, self).__init__(*args, **kwargs)
        self.initialize()

        self.frm_info.collapse_mode = 'mini'
        self.frm_info.split = True

    def set_format_combobox_store(self, enum_cls):
        """
        Подставляет значения в стор комбобокса формата возвращаемого документа
        на основе пришедшего класса перечисления

        @param enum_cls: BaseEnumerate
        @return: None
        """

        self.cmb_format.value = enum_cls.default()
        self.cmb_format.set_store(ExtDataStore(enum_cls.get_choices()))

    def initialize(self):
        '''AUTOGENERATED'''
        self.layout = 'border'
        self.height = 300
        self.width = 620
        self.maximizable = True
        self.template_globals = 'rb-report-form.js'
        self.minimizable = True
        
        tb_buttons = ExtToolBar()
        tb_buttons.layout = 'toolbar'
        
        btn_generate = ExtButton()
        btn_generate.text = u'Сформировать'
        btn_generate.handler = 'submitForm'
        
        btn_close = ExtButton()
        btn_close.text = u'Отмена'
        btn_close.handler = 'function(){ win.close();}'
        
        frm_info = ExtForm()
        frm_info.collapsible = True
        frm_info.layout = 'form'
        frm_info.title = u'Информация о отчете'
        frm_info.region = 'west'
        frm_info.base_cls = 'x-panel'
        frm_info.padding = '5'
        frm_info.header = True
        frm_info.width = 200
        frm_info.label_align = 'top'
        
        pnl_panel_1 = ExtPanel()
        pnl_panel_1.layout = 'fit'
        pnl_panel_1.region = 'center'
        pnl_panel_1.header = False
        
        tab_tappanel_1 = ExtTabPanel()
        tab_tappanel_1.active_tab = 0
        tab_tappanel_1.header = False
        tab_tappanel_1.region = 'south'
        tab_tappanel_1.deferred_render = False
        
        frm_params = ExtForm()
        frm_params.layout = 'form'
        frm_params.title = u'Параметры запроса'
        frm_params.region = 'center'
        frm_params.base_cls = 'x-panel'
        frm_params.padding = '5'
        frm_params.header = True
        frm_params.label_align = 'left'
        
        hdn_report_id = ExtHiddenField()
        hdn_report_id.name = 'report_id'
        
        frm_options = ExtForm()
        frm_options.label_width = 150
        frm_options.title = u'Дополнительные опции'
        frm_options.padding = '5px'
        frm_options.header = True
        frm_options.layout = 'form'
        
        cmb_format = ExtComboBox()
        cmb_format.display_field = 'name'
        cmb_format.label = u'Возвращаемый формат'
        cmb_format.name = 'result_format'
        cmb_format.auto_width = False
        cmb_format.editable = True
        cmb_format.value_field = 'id'
        cmb_format.mode = 'local'
        cmb_format.trigger_action = 'all'
        cmb_format.anchor = '100%'
        
        self.footer_bar = tb_buttons
        
        tb_buttons.items.extend([btn_generate, btn_close])
        frm_params.items.extend([hdn_report_id])
        frm_options.items.extend([cmb_format])
        tab_tappanel_1.items.extend([frm_params, frm_options])
        pnl_panel_1.items.extend([tab_tappanel_1])
        self.items.extend([frm_info, pnl_panel_1])
        
        self.tb_buttons = tb_buttons
        self.btn_generate = btn_generate
        self.btn_close = btn_close
        self.frm_info = frm_info
        self.pnl_panel_1 = pnl_panel_1
        self.tab_tappanel_1 = tab_tappanel_1
        self.frm_params = frm_params
        self.hdn_report_id = hdn_report_id
        self.frm_options = frm_options
        self.cmb_format = cmb_format

        
class ReportData(ExtWindow):

    def __init__(self, params, *args, **kwargs):
        super(ReportData, self).__init__(*args, **kwargs)
        self.initialize()

        grid = ExtMultiGroupinGrid()
        grid.action_data = params['data_action']
        self.grid = grid

        self.items.append(grid)

    def initialize(self):
        '''AUTOGENERATED'''
        self.layout = 'fit'
        self.height = 500
        self.width = 600
        self.maximizable = True
        self.maximized = True
        self.minimizable = True
        
        tb_buttons = ExtToolBar()
        tb_buttons.layout = 'toolbar'
        
        btn_close = ExtButton()
        btn_close.text = u'Закрыть'
        btn_close.handler = 'function(){ win.close();}'
        
        self.footer_bar = tb_buttons
        
        tb_buttons.items.extend([btn_close])
        
        self.tb_buttons = tb_buttons
        self.btn_close = btn_close